# Penrose tiling
#
# rewrote Python code in http://preshing.com/20110831/penrose-tiling-explained
# for Ruby using TurtleField
# Y. Hayakawa, 2013
# 13-02-2019 Python 3

import cmath
import math
from turtle3 import Turtle

GOLDENRATIO = (1 + math.sqrt(5)) / 2

class PenroseTile:
  def __init__(self,t):
      self.turtle = t
      self.triangles = []

  def subdivide(self,triangles):
    result = []
    for color,a,b,c in self.triangles:
      if color == 0:
        p = a + (b - a) / GOLDENRATIO
        result.append([0, c, p, b])
        result.append([1, p, c, a])
      else:
        q = b + (a - b) / GOLDENRATIO
        r = b + (c - b) / GOLDENRATIO
        result.append([1, r, c, a])
        result.append([1, q, r, b])
        result.append([0, r, q, a])
    return result

  def gen(self,level):
    self.triangles=[ ]
    for i in range(10):
      b = cmath.rect(1, (2*i - 1) * math.pi / 10)
      c = cmath.rect(1, (2*i + 1) * math.pi / 10)
      if i % 2 == 0:
        b,c = c,b  

      self.triangles.append([0, 0j, b, c])

    for i in range(level):
      self.triangles = self.subdivide(self.triangles)

  def draw(self,mag):
      self.turtle.clr()
      for color,a,b,c in self.triangles:
          self.turtle.fill()
          if color == 0:
              self.turtle.col(1.0,0.3,0.0)
          else:
              self.turtle.col(0.3,0.2,1.0)

          self.turtle.jump(a.real*mag, a.imag*mag) 
          self.turtle.pd()
          self.turtle.jump(b.real*mag, b.imag*mag) 
          self.turtle.jump(c.real*mag, c.imag*mag) 
          self.turtle.pu()

          self.turtle.col(0.0,0.0,0.0)
          self.turtle.line()

          self.turtle.jump(b.real*mag, b.imag*mag) 
          self.turtle.pd()
          self.turtle.jump(a.real*mag, a.imag*mag) 
          self.turtle.jump(c.real*mag, c.imag*mag) 
          self.turtle.pu()

# main part

ttl = Turtle("localhost")
tile = PenroseTile(ttl)
tile.gen(5)
tile.draw(250)
